// cotaskmem.cs cominterfacewrapper.cs cominterfaces.cs enums.cs cominterfacewrapper.cs TaskTrigger.cs systemtimeclass.cs triggertypeunion.cs
using System;
using System.Collections;
#if !V10 && !V11
using System.Collections.Generic;
#endif
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
#if !V10 && !V11
using System.Runtime.InteropServices.ComTypes;
#endif
using HongliangSoft.Utilities.TaskSchedulers.Interop;
using HongliangSoft.Utilities.Unmanaged;
using HongliangSoft.Utilities.Unmanaged.ComHelpers;

namespace HongliangSoft.Utilities.TaskSchedulers {
	///<summary>^XNACe\܂B</summary>
	public sealed class Task : ComInterfaceWrapper {
		private ITask ITask { get { return (ITask)base.Instance; } }
		///<summary><see cref="ITask"/> C^[tFCXgpăCX^X쐬܂B</summary>
		internal Task(ITask iTask) : base(iTask) {
		}
		///<summary>sAvP[ṼpX擾܂͐ݒ肵܂B</summary>
		///<value>t@Ĉ݂w肵ꍇA^XNXPW[͊ϐɂĒʂĂpX܂B</value>
		///<exception cref="ArgumentNullException">null ݒ肳܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		///<remarks>
		///  <para>Windows  ^XN EBU[h gpēo^ACéA8.3 `̃pXɂȂĂ\܂B</para>
		///</remarks>
		public string ApplicationPath {
			get {
				IntPtr ptr;
				base.CheckHR(this.ITask.GetApplicationName(out ptr));
				using (CoTaskMem mem = ptr) {
					return mem.ReadString(CharSet.Unicode);
				}
			}
			set {
				if (value == null)
					throw new ArgumentNullException("value");
				base.CheckHR(this.ITask.SetApplicationName(value));
			}
		}
		///<summary>sAvP[VɓnR}hC擾܂͐ݒ肵܂B</summary>
		///<value>R}hCgpȂꍇ͋󕶎B<see cref="Task.ApplicationPath"/> st@Cł͂ȂA֘AÂꂽAvP[VŋNꍇAR}hC͖܂BB</value>
		///<exception cref="ArgumentNullException">null ݒ肳܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public string Parameters {
			get {
				IntPtr ptr;
				base.CheckHR(this.ITask.GetParameters(out ptr));
				using (CoTaskMem mem = ptr) {
					return mem.ReadString(CharSet.Unicode);
				}
			}
			set {
				if (value == null)
					throw new ArgumentNullException("value");
				base.CheckHR(this.ITask.SetParameters(value));
			}
		}
		///<summary>sAvP[V̋NfBNg擾܂͐ݒ肵܂B</summary>
		///<value>󕶎̏ꍇANfBNg mstask.exe ݂fBNgɂȂ܂B</value>
		///<exception cref="ArgumentNullException">null ݒ肳܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public string WorkingDirectory {
			get {
				IntPtr ptr;
				base.CheckHR(this.ITask.GetWorkingDirectory(out ptr));
				using (CoTaskMem mem = ptr) {
					DirectoryInfo dir = new DirectoryInfo(mem.ReadString(CharSet.Unicode));
					return dir.FullName;
				}
			}
			set {
				if (value == null)
					throw new ArgumentNullException("value");
				base.CheckHR(this.ITask.SetWorkingDirectory(value));
			}
		}
		///<summary>sAvP[ṼvZXDx擾܂͐ݒ肵܂B</summary>
		///<exception cref="InvalidEnumArgumentException"><see cref="ProcessPriorityClass"/> 񋓑̂ɒ`ĂȂl^܂B</exception>
		///<exception cref="ArgumentOutOfRangeException">l <see cref="ProcessPriorityClass.AboveNormal"/> ܂ <see cref="ProcessPriorityClass.BelowNormal"/> ɐݒ肵܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public ProcessPriorityClass Priority {
			get {
				ProcessPriorityClass priority;
				base.CheckHR(this.ITask.GetPriority(out priority));
				return priority;
			}
			set {
				if (!Enum.IsDefined(typeof(ProcessPriorityClass), value))
					throw new InvalidEnumArgumentException("value", (int)value, typeof(ProcessPriorityClass));
				if (value == ProcessPriorityClass.AboveNormal || value == ProcessPriorityClass.BelowNormal)
					throw new ArgumentOutOfRangeException("value", value, "l AboveNormal ܂ BelowNormal ɐݒ肷邱Ƃ͂ł܂B");
				base.CheckHR(this.ITask.SetPriority(value));
			}
		}
		///<summary>^XNső厞Ԃ擾܂͐ݒ肵܂B</summary>
		///<value>^XN̍ősԂ\A<see cref="TimeSpan"/> ̒lB<see cref="TimeSpan.Zero"/> l̏ꍇAɎs܂B</value>
		///<exception cref="ArgumentOutOfRangeException">^ꂽ <see cref="TimeSpan"/> A4294967295 ~bi悻 49  17  2  47 b) ܂͂ȏw܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public TimeSpan MaxRunTime {
			get {
				const int infini = -1;
				int time;
				base.CheckHR(this.ITask.GetMaxRunTime(out time));
				if (time == infini)
					return new TimeSpan(infini);
				uint milli = unchecked((uint)time);
				return TimeSpan.FromMilliseconds(milli);
			}
			set {
				const int infini = -1;
				if (value < TimeSpan.Zero)
					base.CheckHR(this.ITask.SetMaxRunTime(infini));
				else {
					long time = (long)value.TotalMilliseconds;
					if (time >= 0xffffffff)
						throw new ArgumentOutOfRangeException("value", value, "4294967295 ~bi悻 49  17  2  47 b) ȏɐݒ肷邱Ƃ͂ł܂B");
					base.CheckHR(this.ITask.SetMaxRunTime(unchecked((int)time)));
				}
			}
		}
		///<summary>
		///V^XN̋NgK쐬܂B
		///</summary>
		///<param name="trigger">N\ <see cref="TaskTrigger"/> B</param>
		///<returns>쐬gK̃CfbNXB</returns>
		///<exception cref="ArgumentNullException"><paramref name="trigger"/>  null w肳܂B</exception>
		///<exception cref="ArgumentException"><paramref name="trigger"/> sȒl܂ł܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		///<remarks>
		///  <para>CfbNX̓gKɌŗL̒lł͂ȂA<see cref="DeleteTrigger"/> Ȃǂɂĕϓ̂ŁAɗR[fBOׂł͂܂B</para>
		///</remarks>
		public int CreateTrigger(TaskTrigger trigger) {
			if (trigger == null)
				throw new ArgumentNullException("trigger", "trigger  null ɂ邱Ƃ͂ł܂B");
			short triggerIndex;
			ITaskTrigger iTrigger;
			base.CheckHR(this.ITask.CreateTrigger(out triggerIndex, out iTrigger));
			try {
				base.CheckHR(iTrigger.SetTrigger(trigger));
			}
			finally {
				if (iTrigger != null)
					Marshal.ReleaseComObject(iTrigger);
			}
			return triggerIndex;
		}
		///<summary>
		///w肵CfbNX̃gK폜܂B
		///</summary>
		///<param name="index">폜gK̃CfbNXB</param>
		///<exception cref="ArgumentOutOfRangeException">CfbNX̒l 0 A܂ <see cref="Task.TriggerCount"/> ȏłB</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public void DeleteTrigger(int index) {
			if (index < 0 || index >= this.TriggerCount)
				throw new ArgumentOutOfRangeException("index", index, string.Format("0 ȉA܂݂͌̃^XNACegK ( {0} ) ȏ͎wł܂B", this.TriggerCount));
			unchecked {
				base.CheckHR(this.ITask.DeleteTrigger((short)index));
			}
		}
		///<summary>w肵gK̏擾܂B</summary>
		///<param name="index">擾gK̃CfbNXB</param>
		///<returns>w肵CfbNX̃gK̏B</returns>
		///<exception cref="ArgumentOutOfRangeException">CfbNX̒l 0 A܂ <see cref="Task.TriggerCount"/> ȏłB</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public TaskTrigger GetTrigger(int index) {
			if (index < 0 || index >= this.TriggerCount)
				throw new ArgumentOutOfRangeException("index", index, string.Format("0 ȉA܂݂͌̃^XNACegK ( {0} ) ȏ͎wł܂B", this.TriggerCount));
			ITaskTrigger iTrigger;
			unchecked {
				base.CheckHR(this.ITask.GetTrigger((short)index, out iTrigger));
				try {
					TaskTrigger trigger = new TaskTrigger();
					iTrigger.GetTrigger(trigger);
					return trigger;
				}
				finally {
					if (iTrigger != null)
						Marshal.ReleaseComObject(iTrigger);
				}
			}
		}
		///<summary>
		///w肵CfbNX̃gKAVgKɒu܂B
		///</summary>
		///<param name="index">ugK̃CfbNXB</param>
		///<param name="trigger">VgKB</param>
		///<returns>̃CfbNXɌX݂ĂgKB</returns>
		///<exception cref="ArgumentException"><paramref name="trigger"/> sȒl܂ł܂B</exception>
		///<exception cref="ArgumentNullException"><paramref name="trigger"/>  null w肳܂B</exception>
		///<exception cref="ArgumentOutOfRangeException">CfbNX̒l 0 A܂ <see cref="Task.TriggerCount"/> ȏłB</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public TaskTrigger ReplaceTrigger(int index, TaskTrigger trigger) {
			if (trigger == null)
				throw new ArgumentNullException("trigger", "trigger  null ɂ邱Ƃ͂ł܂B");
			if (index < 0 || index >= this.TriggerCount)
				throw new ArgumentOutOfRangeException("index", index, string.Format("0 ȉA܂݂͌̃^XNACegK ( {0} ) ȏ͎wł܂B", this.TriggerCount));
			ITaskTrigger iTrigger;
			unchecked {
				base.CheckHR(this.ITask.GetTrigger((short)index, out iTrigger));
				try {
					TaskTrigger oldTrigger = new TaskTrigger();
					iTrigger.GetTrigger(oldTrigger);
					iTrigger.SetTrigger(trigger);
					return oldTrigger;
				}
				finally {
					if (iTrigger != null)
						Marshal.ReleaseComObject(iTrigger);
				}
			}
		}
		///<summary>w肵gK̕\擾܂B</summary>
		///<param name="index">擾gK̃CfbNXB</param>
		///<returns>gK̕\B</returns>
		///<exception cref="ArgumentOutOfRangeException">CfbNX̒l 0 A܂ <see cref="Task.TriggerCount"/> ȏłB</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public string GetTriggerDescription(int index) {
			if (index < 0 || index >= this.TriggerCount)
				throw new ArgumentOutOfRangeException("index", index, string.Format("0 ȉA܂݂͌̃^XNACegK ( {0} ) ȏ͎wł܂B", this.TriggerCount));
			IntPtr ptr;
			unchecked {
				base.CheckHR(this.ITask.GetTriggerString((short)index, out ptr));
				using (CoTaskMem mem = ptr) {
					return mem.ReadString(CharSet.Unicode);
				}
			}
		}
		///<summary>̃^XNACeɊ֘AÂꂽSẴgK̏擾܂B</summary>
		///<returns>egK̏i[ <see cref="TaskTrigger"/> zBz̈ʒúAĂяo_ł̃^XNACẽgK̃CfbNXɈv܂B</returns>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public TaskTrigger[] GetTriggers() {
			ushort count = (ushort)this.TriggerCount;
			TaskTrigger[] triggers = new TaskTrigger[count];
			for (short i = 0; i < count; i++) {
				triggers[i] = this.GetTrigger(i);
			}
			return triggers;
		}
		///<summary>^XNɊ֘AÂꂽgK̑擾܂B</summary>
		///<value></value>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public int TriggerCount {
			get {
				short count;
				base.CheckHR(this.ITask.GetTriggerCount(out count));
				return count;
			}
		}
		///<summary>x <see cref="GetRunTimes(DateTime, DateTime, int)"/> \bȟĂяoŎ擾łő̗vfłB</summary>
		public static readonly int RunTimesMax = 1440;
		///<summary>
		///^XNN̔z擾܂B擾Ώۂ́A݂̓ȍ~̑SĂłB
		///</summary>
		///<param name="maxCount">擾z̗vf̍ő吔B</param>
		///<returns>^XNN̔zBK <paramref name="maxCount"/> ̗vfłƂ͌܂BNȂꍇAvf 0 ̔złB</returns>
		///<exception cref="ArgumentOutOfRangeException"><paramref name="maxCount"/>  0  0 A܂ <see cref="RunTimesMax"/> 𒴂Ă܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public DateTime[] GetRunTimes(int maxCount) {
			return this.GetRunTimes(DateTime.Now, DateTime.MaxValue, maxCount);
		}
		///<summary>
		///^XNN̔z擾܂B擾Ώۂ́A݂̓ȍ~Aw肵܂łłB
		///</summary>
		///<param name="endTime">N̎擾̓B</param>
		///<returns>^XNN̔zBvf͍ő <see cref="RunTimesMax"/> ܂łłB</returns>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		///<remarks>
		///  <para>ȉ̏󋵂ł͗vf 0 ̔z񂪕Ԃ܂B
		///    <list type="bullet">
		///      <item><description>NȂB</description></item>
		///      <item><description><paramref name="endTime"/> Ɍ݂̓ȑO̓w肵B</description></item>
		///    </list>
		///  </para>
		///</remarks>
		public DateTime[] GetRunTimes(DateTime endTime) {
			return this.GetRunTimes(DateTime.Now, endTime);
		}
		///<summary>
		///^XNN̔z擾܂B擾Ώۂ́Aw肵ȍ~Aw肵܂łłB
		///</summary>
		///<param name="beginTime">N̎擾̓B</param>
		///<param name="endTime">N̎擾̓B</param>
		///<returns>^XNN̔zBvf͍ő <see cref="RunTimesMax"/> ܂łłB</returns>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		///<remarks>
		///  <para>ȉ̏󋵂ł͗vf 0 ̔z񂪕Ԃ܂B
		///    <list type="bullet">
		///      <item><description>NȂB</description></item>
		///      <item><description><paramref name="endTime"/>  <paramref name="startTime"/> ȑO̓w肵B</description></item>
		///      <item><description><paramref name="startTime"/>  <paramref name="endTime"/> ̂ꂩ܂͗ɁA1601 N 1  1  ȑO̓w肵B</description></item>
		///    </list>
		///  </para>
		///</remarks>
		public DateTime[] GetRunTimes(DateTime beginTime, DateTime endTime) {
			return this.GetRunTimes(beginTime, endTime, Task.RunTimesMax);
		}
		///<summary>
		///^XNN̔z擾܂B擾Ώۂ́Aw肵ȍ~Aw肵܂łłB
		///</summary>
		///<param name="beginTime">N̎擾̓B</param>
		///<param name="endTime">N̎擾̓B</param>
		///<param name="maxCount">擾z̗vf̍ő吔B</param>
		///<returns>^XNN̔zBK <paramref name="maxCount"/> ̗vfłƂ͌܂B</returns>
		///<exception cref="ArgumentOutOfRangeException"><paramref name="maxCount"/>  0  0 A܂<see cref="RunTimesMax"/> 𒴂Ă܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		///<remarks>
		///  <para>ȉ̏󋵂ł͗vf 0 ̔z񂪕Ԃ܂B
		///    <list type="bullet">
		///      <item><description>NȂB<see cref="TaskTrigger.SetAsOnIdle"/> ĝȂǃCxg쓮^CṽgKĂȂAgKĂȂꍇ܂܂܂B</description></item>
		///      <item><description><paramref name="endTime"/>  <paramref name="startTime"/> ȑO̓w肵B</description></item>
		///      <item><description><paramref name="startTime"/>  <paramref name="endTime"/> ̂ꂩ܂͗ɁA1601 N 1  1  ȑO̓w肵B</description></item>
		///    </list>
		///  </para>
		///</remarks>
		public DateTime[] GetRunTimes(DateTime beginTime, DateTime endTime, int maxCount) {
			if (maxCount < 1 || maxCount > Task.RunTimesMax)
				throw new ArgumentOutOfRangeException("maxCount", maxCount, "0 ȉA܂ Task.RunTimesMax 傫lw肷邱Ƃ͂ł܂B");
			SystemTimeClass begin = new SystemTimeClass(beginTime), end = new SystemTimeClass(endTime);
			short count = (short)maxCount;
			IntPtr runTimes;
			HResult result = this.ITask.GetRunTimes(begin, end, ref count, out runTimes);
			if (result != HResult.OK && result != HResult.False) {
				CheckHR(result);
				return new DateTime[0];
			}
			using (CoTaskMem array = runTimes) {
#if !V10 && !V11
				List<DateTime> dates = new List<DateTime>();
#else
				ArrayList dates = new ArrayList();
#endif
				int size = Marshal.SizeOf(typeof(SystemTimeClass));
				for (int i = 0; i < count; i++) {
					SystemTimeClass runs;
#if !V10 && !V11
					runs = array.ReadStructure<SystemTimeClass>(size * i);
#else
					runs = (SystemTimeClass)array.ReadStructure(typeof(SystemTimeClass), size * i);
#endif
					if (! runs.IsInvalid) {
						dates.Add(runs.ToDateTime());
					}
				}
#if !V10 && !V11
				return dates.ToArray();
#else
				return (DateTime[])dates.ToArray(typeof(DateTime));
#endif
			}
		}
		///<summary>
		///̋N擾܂B
		///</summary>
		///<returns>̋NBNȂꍇ́A<see cref="DateTime.MinValue"/> Ԃ܂B</returns>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public DateTime GetNextRunTime() {
			DateTime result;
			return (TryGetNextRunTime(out result)) ? result : DateTime.MinValue;
		}
		///<summary>
		///̋N擾܂B
		///</summary>
		///<param name="result">䂪ԂƂɁANi[邽߂ <see cref="DateTime"/> ̎QƁB̃\bh false ԂƂ̒l͖`łB</param>
		///<returns>̋N݂ꍇ true B</returns>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public bool TryGetNextRunTime(out DateTime result) {
			SystemTimeClass time = SystemTimeClass.MinValue;
			base.CheckHR(this.ITask.GetNextRunTime(time));
			if (time.IsInvalid) {
				result = DateTime.MinValue;
				return false;
			}
			else {
				result = time.ToDateTime();
				return true;
			}
		}
		///<summary>^XN̋NɕKvȃAChԂ擾܂͐ݒ肵܂B</summary>
		///<value>NɕKvȃAChԂ\A<see cref="IdleWait"/> ̒lB</value>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public IdleWait IdleWait {
			get {
				short idle, deadline;
				base.CheckHR(this.ITask.GetIdleWait(out idle, out deadline));
				unchecked {
					return new IdleWait((ushort)idle, (ushort)deadline);
				}
			}
			set {
				unchecked {
					short idle = (short)value.IdleMinutes;
					short deadline = (short)value.DeadlineMinutes;
					base.CheckHR(this.ITask.SetIdleWait(idle, deadline));
				}
			}
		}
		///<summary>
		///^XN̋N^XNXPW[ɗv܂B
		///</summary>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		///<remarks>
		///  <para>̃\bh͔񓯊ōs܂BۂɃ^XNNɂ炸䂪Ԃ̂ŁAႦΉbxă^XNN\܂B</para>
		///  <para>^XNACȅԂmɂ <see cref="Task.Status"/> gpĂB</para>
		///</remarks>
		public void Run() {
			base.CheckHR(this.ITask.Run());
		}
		///<summary>
		///s̃^XÑAvP[VI܂B
		///</summary>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		///<remarks>
		///  <para>̃\bh͔񓯊ōs܂BvȌI҂Ƃ͂܂B</para>
		///  <para>^XNXPW[́AÎɑΏۂ̃vÕEBhE WM_CLOSE 𑗐M܂BbZ[W󂯎EBhEȂꍇA܂ WM_CLOSE 𑗐MĂ 3 o߂ĂIȂꍇATerminateProcess ֐gpċIɏI܂B</para>
		///</remarks>
		public void Terminate() {
			base.CheckHR(this.ITask.Terminate());
		}
		///<summary>
		///^XNACẽvpeB_CAOJ܂B
		///</summary>
		///<param name="owner">_CAÕI[i[ƂȂtH[BI[i[Ȃꍇ null w肵܂B</param>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		///<remarks>
		///  <para>̃\bhĂяo܂ł <see cref="Task"/> ɉύX́AIɕۑAύX̒lŃ_CAOJ܂B</para>
		///</remarks>
		public void ShowPropertyPage(System.Windows.Forms.Form owner) {
			IntPtr window = (owner == null) ? IntPtr.Zero : owner.Handle;
			base.CheckHR(this.ITask.EditWorkItem(window, 0));
		}
		///<summary>
		///߂Ƀ^XNN擾܂B
		///</summary>
		///<returns>߂Ƀ^XNNBxNƂȂꍇA<see cref="DateTime.MinValue"/> Ԃ܂B</returns>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public DateTime GetMostRecentRunTime() {
			DateTime result;
			return (TryGetMostRecentRunTime(out result)) ? result : DateTime.MinValue;
		}
		///<summary>
		///߂Ƀ^XNN擾܂B
		///</summary>
		///<param name="result">䂪ԂƂɁA߂ɋNi[邽߂ <see cref="DateTime"/> ̎QƁB̃\bh false ԂƂ̒l͖`łB</param>
		///<returns>߂̋N݂ꍇ true BxNƂȂꍇ false B</returns>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public bool TryGetMostRecentRunTime(out DateTime result) {
			SystemTimeClass time = SystemTimeClass.MinValue;
			base.CheckHR(this.ITask.GetMostRecentRunTime(time));
			if (time.IsInvalid) {
				result = DateTime.MinValue;
				return false;
			}
			else {
				result = time.ToDateTime();
				return true;
			}
		}
		///<summary>
		///^XNȑOɋNƂ邩ǂ擾܂B
		///</summary>
		///<returns>ȑOɋNƂꍇ true B</returns>
		public bool HadRun() {
			SystemTimeClass time = SystemTimeClass.MinValue;
			base.CheckHR(this.ITask.GetMostRecentRunTime(time));
			return !(time.IsInvalid);
		}
		///<summary>^XNACȅԂ擾܂B</summary>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public TaskStatus Status {
			get {
				TaskStatus status;
				base.CheckHR(this.ITask.GetStatus(out status));
				return status;
			}
		}
		///<summary>^XNACesAvP[V̒ŁAŌɎsꂽ^XN̏IR[h擾܂B</summary>
		///<returns>^XN̏IR[hBxNĂȂꍇ 0 B</returns>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public int GetExitCode() {
			if (!this.HadRun())
				return 0;
			int code;
			base.CheckHR(this.ITask.GetExitCode(out code));
			return code;
		}
		///<summary>^XNACẽRg擾܂͐ݒ肵܂B</summary>
		///<exception cref="ArgumentNullException">null w肳܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public string Comment {
			get {
				IntPtr ptr;
				base.CheckHR(this.ITask.GetComment(out ptr));
				using (CoTaskMem mem = ptr) {
					return mem.ReadString(CharSet.Unicode);
				}
			}
			set {
				if (value == null)
					throw new ArgumentNullException("value", "Rg null ɂ邱Ƃ͂ł܂B");
				base.CheckHR(this.ITask.SetComment(value));
			}
		}
		///<summary>쐬҂\擾܂͐ݒ肵܂B</summary>
		///<exception cref="ArgumentNullException">null w肳܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public string Creator {
			get {
				IntPtr ptr;
				base.CheckHR(this.ITask.GetCreator(out ptr));
				using (CoTaskMem mem = ptr) {
					return mem.ReadString(CharSet.Unicode);
				}
			}
			set {
				if (value == null)
					throw new ArgumentNullException("value", "Creator  null ɂ邱Ƃ͂ł܂B");
				base.CheckHR(this.ITask.SetCreator(value));
			}
		}
		///<summary>AvP[V`̒ǉf[^擾܂͐ݒ肵܂B</summary>
		///<value>Őݒ肵f[^́A^XNXPW[̋ɂ͈؉e܂B</value>
		///<exception cref="ArgumentException">f[^TCY 65535 oCg𒴂܂B</exception>
		///<exception cref="ArgumentNullException">null w肵܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public byte[] OptionalData {
			get {
				IntPtr ptr;
				short size;
				base.CheckHR(this.ITask.GetWorkItemData(out size, out ptr));
				using (CoTaskMem mem = ptr) {
					return mem.ReadBytes(size);
				}
			}
			set {
				if (value == null)
					throw new ArgumentNullException();
				if (value.Length > 65535)
					throw new ArgumentException();
				unchecked {
					short size = (short)value.Length;
					base.CheckHR(this.ITask.SetWorkItemData(size, value));
				}
			}
		}
/*
		///<summary>݃T|[gĂ܂BK <see cref="NotSupportedException"/> ܂B</summary>
		///<exception cref="NotSupportedException">̃vpeBɑ΂Ăяo܂͐ݒs܂B</exception>
		public int ErrorRetryCount {
			get { throw new NotSupportedException("݃T|[gĂ܂B"); }
			set { throw new NotSupportedException("݃T|[gĂ܂B"); }
		}
		///<summary>݃T|[gĂ܂BK <see cref="NotSupportedException"/> ܂B</summary>
		///<exception cref="NotSupportedException">̃vpeBɑ΂Ăяo܂͐ݒs܂B</exception>
		public int ErrorRetryInterval {
			get { throw new NotSupportedException("݃T|[gĂ܂B"); }
			set { throw new NotSupportedException("݃T|[gĂ܂B"); }
		}
*/
		///<summary>^XNɊւtO擾܂͐ݒ肵܂B</summary>
		///<value>^XN̋ɊւetO\ <see cref="TaskFlags"/> ̑gݍ킹B</value>
		///<exception cref="InvalidEnumArgumentException">s <see cref="TaskFlags"/> w肵܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public TaskFlags Flags {
			get {
				TaskFlags flags;
				base.CheckHR(this.ITask.GetFlags(out flags));
				return flags;
			}
			set {
				if ((int)value < 0 || (int)value > 0x3FFF)
					throw new InvalidEnumArgumentException("value", (int)value, typeof(TaskFlags));
				base.CheckHR(this.ITask.SetFlags(value));
			}
		}
		///<summary>
		///^XNsAJEgݒ肵܂B
		///</summary>
		///<value></value>
		///<exception cref="ArgumentException"><paramref name="accountName"/> ɋ󕶎w肵ĂƂɁA<paramref name="password"/>  null ȊOw肵܂B</exception>
		///<exception cref="ArgumentNullException"><paramref name="accountName"/>  null w肵܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		///<remarks>
		///  <para><see cref="Task.Flags"/>  <see cref="TaskFlags.RunOnlyIfLoggedOn"/> w肵ĂꍇA<paramref name="password"/>  null wł܂BɂAAvP[V̓[ŨpX[hmKvȂ^XNo^ł܂BAA̕@œo^ꍇA[UOIĂȂƃ^XNN܂B</para>
		///  <para>݂ȂAJEgA܂͌pX[hݒ肵ꍇA<see cref="Task.Save"/> \bhĂяoۂɁAAJEg݂Ȃꍇ͗O <see cref="COMException"/>  ErrorCode 0x80041310 ƂāA܂pX[hݒ肵ꍇ͗O <see cref="UnauthorizedAccessException"/> ܂B</para>
		///</remarks>
		///<seealso cref="Environment.UserName"/>
		public void SetAccount(string accountName, string password) {
			if (accountName == null)
				throw new ArgumentNullException("accountName", "AJEg null ɂ邱Ƃ͂ł܂B");
			if (accountName == "" && password != null)
				throw new ArgumentException("", "accountName 󕶎ɂꍇApassword  null łȂ΂Ȃ܂B");
			base.CheckHR(this.ITask.SetAccountInformation(accountName, password));
		}
		///<summary>
		///^XNsAJEg̖O擾܂B
		///</summary>
		///<returns>^XNsAJEgB</returns>
		///<value></value>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public string GetAccount() {
			IntPtr ptr;
			base.CheckHR(this.ITask.GetAccountInformation(out ptr));
			using (CoTaskMem mem = ptr) {
				return mem.ReadString(CharSet.Unicode);
			}
		}
		///<summary>
		///<see cref="Task"/> ɉύXۑ܂B
		///</summary>
		///<exception cref="UnauthorizedAccessException">ݒ肵AJEgɑ΂ăpX[hłB</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public void Save() {
			IPersistFile persist = (IPersistFile)this.ITask;
			persist.Save(null, true);
		}
	}
	///<summary>^XNNOɕKvȃAChԂ\܂B</summary>
	///<remarks>
	///  <para><see cref="TaskTrigger"/>  <see cref="TaskTrigger.SetAsOnIdle"/> ĂяoƂɂāÃgKAChgKł邱Ƃݒł܂BAChgKƂ́AԎwx[X̃gKł͂ȂCxgx[X̃gKłBAChgḰAVXe <see cref="IdleMinutes"/> Ŏw肳ꂽԃ[U̓͂ȂԁiAChԁjꍇɋN܂B̂ƂA<see cref="DeadlineMinutes"/> ̒l͖܂B</para>
	///  <para>L̃AChgK̏ꍇ̂قAԎwx[X̃gKɂӖ܂B<see cref="Task.Flags"/>  <see cref="TaskFlags.StartOnlyIfIdle"/> ݒ肷ƁAgK̎w肵ɂȂƂA^XN <see cref="IdleMinutes"/> Ŏw肳ꂽԈȏAChԂɂȂĂꍇ̂݋N悤ɂȂ܂B^XNXPW[́A<see cref="IdleMinutes"/> ɂă^XNNȂꍇA^XNACe̊Jn\莞 <paramref name="IdleWait.DeadlineMinutes"/> Ŏw肳ꂽԂҋ@AVXeAChԂɂȂ邩`FbN܂B <paramref name="DeadlineMinutes"/> ̊Ԃ <paramref name="IdleMinutes"/> 𖞂ƂłȂA^XNXPW[͂̃^C~OŃ^XNN܂B</para>
	///</remarks>
	public struct IdleWait {
		private int idleMinutes;
		private int deadlineMinutes;
		///<summary>NOɕKvȃAChԂ擾܂͐ݒ肵܂B</summary>
		///<value>NOɕKvȁAPʂ̃AChԁB</value>
		///<exception cref="ArgumentOutOfRangeException">0 lA܂ 65535 傫lɐݒ肵܂B</exception>
		public int IdleMinutes {
			get { return this.idleMinutes; }
			set {
				if (value < 0 || value > 65535)
					throw new ArgumentOutOfRangeException("value", value, "0 lA܂ 65535 傫lɂ邱Ƃ͂ł܂B");
				this.idleMinutes = value;
			}
		}
		///<summary>NOɕKvȃAChԂ擾܂͐ݒ肵܂B</summary>
		///<value>NOɕKvȃAChԂ\ <see cref="TimeSpan"/> B</value>
		///<exception cref="ArgumentOutOfRangeException">}CiX̒lA܂ 65535 Ԃ\l傫lɐݒ肵܂B</exception>
		public TimeSpan IdleTime {
			get { return TimeSpan.FromMinutes(this.idleMinutes); }
			set {
				long minutes = (long)value.TotalMinutes;
				if (minutes < 0 || minutes > 65535)
					throw new ArgumentOutOfRangeException("value", value, "}CiX̒lA܂ 65535 Ԃ\l傫lɂ邱Ƃ͂ł܂B");
				this.idleMinutes = (int)minutes;
			}
		}
		///<summary>NɃAChԂȂꍇ́A^XNXPW[̑ҋ@Ԃ擾܂͐ݒ肵܂B</summary>
		///<value>Pʂ̃^XNXPW[̑ҋ@ԁB</value>
		///<exception cref="ArgumentOutOfRangeException">0 lA܂ 65535 傫lɐݒ肵܂B</exception>
		public int DeadlineMinutes {
			get { return this.deadlineMinutes; }
			set {
				if (value < 0 || value > 65535)
					throw new ArgumentOutOfRangeException("value", value, "0 lA܂ 65535 傫lɂ邱Ƃ͂ł܂B");
				this.deadlineMinutes = value;
			}
		}
		///<summary>NɃAChԂȂꍇ́A^XNXPW[̑ҋ@Ԃ擾܂͐ݒ肵܂B</summary>
		///<value>^XNXPW[̑ҋ@Ԃ\ <see cref="TimeSpan"/> B</value>
		///<exception cref="ArgumentOutOfRangeException">}CiX̒lA܂ 65535 Ԃ\l傫lɐݒ肵܂B</exception>
		public TimeSpan DeadlineTime {
			get { return TimeSpan.FromMinutes(this.deadlineMinutes); }
			set {
				long minutes = (long)value.TotalMinutes;
				if (minutes < 0 || minutes > 65535)
					throw new ArgumentOutOfRangeException("value", value, "}CiX̒lA܂ 65535 Ԃ\l傫lɂ邱Ƃ͂ł܂B");
				this.deadlineMinutes = (int)minutes;
			}
		}
		///<summary>
		///NOɕKvȃAChԂƁANɃAChԂȂƂ̃^XNXPW[̑ҋ@Ԃw肵ăCX^X܂B
		///</summary>
		///<param name="idleMinutes">NɕKvȁAPʂ̃AChԁB</param>
		///<param name="deadlineMinutes">Pʂ̃^XNXPW[̑ҋ@ԁB</param>
		///<exception cref="ArgumentOutOfRangeException"><paramref name="idleMinutes"/> A܂ <paramref name="deadlineMintes"/> A0 lA܂ 65535 傫lɐݒ肵܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public IdleWait(int idleMinutes, int deadlineMinutes) {
			if (idleMinutes < 0 || idleMinutes > 65535)
				throw new ArgumentOutOfRangeException("idleMinutes", idleMinutes, "idleMinutes  0 lA܂ 65535 傫lɂ邱Ƃ͂ł܂B");
			if (deadlineMinutes < 0 || deadlineMinutes > 65535)
				throw new ArgumentOutOfRangeException("deadlineMinutes", deadlineMinutes, "deadlineMinutes  0 lA܂ 65535 傫lɂ邱Ƃ͂ł܂B");
			this.idleMinutes = idleMinutes;
			this.deadlineMinutes = deadlineMinutes;
		}
		///<summary>
		///NOɕKvȃAChԂƁANɃAChԂȂƂ̃^XNXPW[̑ҋ@Ԃw肵ăCX^X܂B
		///</summary>
		///<param name="idleTime">NɕKvȁAPʂ̃AChԁB</param>
		///<param name="deadlineTime">Pʂ̃^XNXPW[̑ҋ@ԁB</param>
		///<exception cref="ArgumentOutOfRangeException"><paramref name="idleTime"/> A܂ <paramref name="deadlineTime"/> A0 lA܂ 65535 傫lɐݒ肵܂B</exception>
		///<exception cref="COMException">FȂ HRESULT Ԃ܂B</exception>
		public IdleWait(TimeSpan idleTime, TimeSpan deadlineTime) {
			long idle = (long)idleTime.TotalMinutes;
			if (idle < 0 || idle > 65535)
				throw new ArgumentOutOfRangeException("idleTime", idleTime, "}CiX̒lA܂ 65535 Ԃ\l傫lɂ邱Ƃ͂ł܂B");
			long deadline = (long)deadlineTime.TotalMinutes;
			if (deadline < 0 || deadline > 65535)
				throw new ArgumentOutOfRangeException("deadlineTime", deadlineTime, "deadlineMinutes  0 lA܂ 65535 傫lɂ邱Ƃ͂ł܂B");
			this.idleMinutes = (int)idle;
			this.deadlineMinutes = (int)deadline;
		}
	}
}
